@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -171,17 +171,8 @@
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Load ACPI/AML table by executing the entire table as a single
- *              large control method.
- *
- * NOTE: The point of this is to execute any module-level code in-place
- * as the table is parsed. Some AML code depends on this behavior.
- *
- * It is a run-time option at this time, but will eventually become
- * the default.
- *
- * Note: This causes the table to only have a single-pass parse.
- * However, this is compatible with other ACPI implementations.
+ * DESCRIPTION: Load ACPI/AML table by executing the entire table as a
+ *              TermList.
  *
  ******************************************************************************/
 
@@ -261,18 +252,8 @@ AcpiNsExecuteTable (
         goto Cleanup;
     }
 
-    /* Optional object evaluation log */
-
-    ACPI_DEBUG_PRINT_RAW ((ACPI_DB_EVALUATION,
-        "%-26s:  (Definition Block level)\n", "Module-level evaluation"));
-
     Status = AcpiPsExecuteTable (Info);
 
-    /* Optional object evaluation log */
-
-    ACPI_DEBUG_PRINT_RAW ((ACPI_DB_EVALUATION,
-        "%-26s:  (Definition Block level)\n", "Module-level complete"));
-
 Cleanup:
     if (Info)
     {
@@ -422,19 +403,8 @@ AcpiNsParseTable (
     ACPI_FUNCTION_TRACE (NsParseTable);
 
 
-    if (AcpiGbl_ExecuteTablesAsMethods)
+    if (AcpiGbl_ParseTableAsTermList)
     {
-        /*
-         * This case executes the AML table as one large control method.
-         * The point of this is to execute any module-level code in-place
-         * as the table is parsed. Some AML code depends on this behavior.
-         *
-         * It is a run-time option at this time, but will eventually become
-         * the default.
-         *
-         * Note: This causes the table to only have a single-pass parse.
-         * However, this is compatible with other ACPI implementations.
-         */
         ACPI_DEBUG_PRINT_RAW ((ACPI_DB_PARSE,
             "%s: **** Start table execution pass\n", ACPI_GET_FUNCTION_NAME));
 
