@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2015, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,6 +111,42 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #include "aecommon.h"
@@ -159,7 +195,6 @@ static ACPI_TABLE_XSDT          *LocalXSDT;
 #define BASE_XSDT_SIZE          ((BASE_XSDT_TABLES) * sizeof (UINT64))
 
 #define ACPI_MAX_INIT_TABLES    (32)
-static ACPI_TABLE_DESC          Tables[ACPI_MAX_INIT_TABLES];
 
 
 /******************************************************************************
@@ -224,10 +259,16 @@ AeInitializeTableHeader (
     Header->Length = Length;
 
     Header->OemRevision = 0x1001;
-    ACPI_STRNCPY (Header->OemId, "Intel", ACPI_OEM_ID_SIZE);
-    ACPI_STRNCPY (Header->OemTableId, "AcpiExec", ACPI_OEM_TABLE_ID_SIZE);
-    ACPI_STRNCPY (Header->AslCompilerId, "INTL", ACPI_NAME_SIZE);
-    Header->AslCompilerRevision = 0x20131218;
+    strncpy (Header->OemId, "Intel", ACPI_OEM_ID_SIZE);
+    strncpy (Header->OemTableId, "AcpiExec", ACPI_OEM_TABLE_ID_SIZE);
+    strncpy (Header->AslCompilerId, "INTL", ACPI_NAME_SIZE);
+    Header->AslCompilerRevision = ACPI_CA_VERSION;
+
+    /* Set the checksum, must set to zero first */
+
+    Header->Checksum = 0;
+    Header->Checksum = (UINT8) -AcpiTbChecksum (
+        (void *) Header, Header->Length);
 }
 
 
@@ -236,7 +277,7 @@ AeInitializeTableHeader (
  * FUNCTION:    AeBuildLocalTables
  *
  * PARAMETERS:  TableCount      - Number of tables on the command line
- *              TableList       - List of actual tables from files
+ *              ListHead        - List of actual tables from files
  *
  * RETURN:      Status
  *
@@ -247,12 +288,12 @@ AeInitializeTableHeader (
 
 ACPI_STATUS
 AeBuildLocalTables (
-    UINT32                  TableCount,
-    AE_TABLE_DESC           *TableList)
+    ACPI_NEW_TABLE_DESC     *ListHead)
 {
+    UINT32                  TableCount = 1;
     ACPI_PHYSICAL_ADDRESS   DsdtAddress = 0;
     UINT32                  XsdtSize;
-    AE_TABLE_DESC           *NextTable;
+    ACPI_NEW_TABLE_DESC     *NextTable;
     UINT32                  NextIndex;
     ACPI_TABLE_FADT         *ExternalFadt = NULL;
 
@@ -262,18 +303,20 @@ AeBuildLocalTables (
      * For the FADT, this table is already accounted for since we usually
      * install a local FADT.
      */
-    NextTable = TableList;
+    NextTable = ListHead;
     while (NextTable)
     {
-        if (ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_DSDT) ||
-            ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_FADT))
+        if (!ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_DSDT) &&
+            !ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_FADT))
         {
-            TableCount--;
+            TableCount++;
         }
+
         NextTable = NextTable->Next;
     }
 
-    XsdtSize = (((TableCount + 1) * sizeof (UINT64)) + sizeof (ACPI_TABLE_HEADER));
+    XsdtSize = (((TableCount + 1) * sizeof (UINT64)) +
+        sizeof (ACPI_TABLE_HEADER));
     if (AcpiGbl_LoadTestTables)
     {
         XsdtSize += BASE_XSDT_SIZE;
@@ -287,20 +330,15 @@ AeBuildLocalTables (
         return (AE_NO_MEMORY);
     }
 
-    ACPI_MEMSET (LocalXSDT, 0, XsdtSize);
-    AeInitializeTableHeader ((void *) LocalXSDT, ACPI_SIG_XSDT, XsdtSize);
-
+    memset (LocalXSDT, 0, XsdtSize);
     LocalXSDT->TableOffsetEntry[0] = ACPI_PTR_TO_PHYSADDR (&LocalFADT);
     NextIndex = 1;
 
     /*
      * Install the user tables. The DSDT must be installed in the FADT.
      * All other tables are installed directly into the XSDT.
-     *
-     * Note: The tables are loaded in reverse order from the incoming
-     * input, which makes it match the command line order.
      */
-    NextTable = TableList;
+    NextTable = ListHead;
     while (NextTable)
     {
         /*
@@ -329,7 +367,7 @@ AeBuildLocalTables (
         {
             /* Install the table in the XSDT */
 
-            LocalXSDT->TableOffsetEntry[TableCount - NextIndex + 1] =
+            LocalXSDT->TableOffsetEntry[NextIndex] =
                 ACPI_PTR_TO_PHYSADDR (NextTable->Table);
             NextIndex++;
         }
@@ -369,9 +407,9 @@ AeBuildLocalTables (
 
     /* Build an RSDP. Contains a valid XSDT only, no RSDT */
 
-    ACPI_MEMSET (&LocalRSDP, 0, sizeof (ACPI_TABLE_RSDP));
+    memset (&LocalRSDP, 0, sizeof (ACPI_TABLE_RSDP));
     ACPI_MAKE_RSDP_SIG (LocalRSDP.Signature);
-    ACPI_MEMCPY (LocalRSDP.OemId, "Intel", 6);
+    memcpy (LocalRSDP.OemId, "Intel", 6);
 
     LocalRSDP.Revision = 2;
     LocalRSDP.XsdtPhysicalAddress = ACPI_PTR_TO_PHYSADDR (LocalXSDT);
@@ -379,9 +417,7 @@ AeBuildLocalTables (
 
     /* Set checksums for both XSDT and RSDP */
 
-    LocalXSDT->Header.Checksum = 0;
-    LocalXSDT->Header.Checksum = (UINT8) -AcpiTbChecksum (
-        (void *) LocalXSDT, LocalXSDT->Header.Length);
+    AeInitializeTableHeader ((void *) LocalXSDT, ACPI_SIG_XSDT, XsdtSize);
 
     LocalRSDP.Checksum = 0;
     LocalRSDP.Checksum = (UINT8) -AcpiTbChecksum (
@@ -401,31 +437,38 @@ AeBuildLocalTables (
      * 2) A "hardware reduced" local FADT
      * 3) A fully featured local FADT
      */
+    memset (&LocalFADT, 0, sizeof (ACPI_TABLE_FADT));
+
     if (ExternalFadt)
     {
         /*
-         * Use the external FADT, but we must update the DSDT/FACS addresses
-         * as well as the checksum
+         * Use the external FADT, but we must update the DSDT/FACS
+         * addresses as well as the checksum
          */
-        ExternalFadt->Dsdt = DsdtAddress;
+        ExternalFadt->Dsdt = (UINT32) DsdtAddress;
         if (!AcpiGbl_ReducedHardware)
         {
             ExternalFadt->Facs = ACPI_PTR_TO_PHYSADDR (&LocalFACS);
         }
 
-        /* Is there room in the FADT for the XDsdst and XFacs 64-bit pointers? */
-
-        if (ExternalFadt->Header.Length > ACPI_PTR_DIFF (&ExternalFadt->XDsdt, ExternalFadt))
+        /*
+         * If there room in the FADT for the XDsdt and XFacs 64-bit
+         * pointers, use them.
+         */
+        if (ExternalFadt->Header.Length > ACPI_PTR_DIFF (
+            &ExternalFadt->XDsdt, ExternalFadt))
         {
-            ExternalFadt->XDsdt = DsdtAddress;
+            ExternalFadt->Dsdt = 0;
+            ExternalFadt->Facs = 0;
 
+            ExternalFadt->XDsdt = DsdtAddress;
             if (!AcpiGbl_ReducedHardware)
             {
                 ExternalFadt->XFacs = ACPI_PTR_TO_PHYSADDR (&LocalFACS);
             }
         }
 
-        /* Complete the FADT with the checksum */
+        /* Complete the external FADT with the checksum */
 
         ExternalFadt->Header.Checksum = 0;
         ExternalFadt->Header.Checksum = (UINT8) -AcpiTbChecksum (
@@ -433,22 +476,16 @@ AeBuildLocalTables (
     }
     else if (AcpiGbl_UseHwReducedFadt)
     {
-        ACPI_MEMCPY (&LocalFADT, HwReducedFadtCode, sizeof (ACPI_TABLE_FADT));
-        LocalFADT.Dsdt = DsdtAddress;
+        memcpy (&LocalFADT, HwReducedFadtCode, ACPI_FADT_V5_SIZE);
+        LocalFADT.Dsdt = 0;
         LocalFADT.XDsdt = DsdtAddress;
-
-        LocalFADT.Header.Checksum = 0;
-        LocalFADT.Header.Checksum = (UINT8) -AcpiTbChecksum (
-            (void *) &LocalFADT, LocalFADT.Header.Length);
     }
     else
     {
         /*
          * Build a local FADT so we can test the hardware/event init
          */
-        ACPI_MEMSET (&LocalFADT, 0, sizeof (ACPI_TABLE_FADT));
         LocalFADT.Header.Revision = 5;
-        AeInitializeTableHeader ((void *) &LocalFADT, ACPI_SIG_FADT, sizeof (ACPI_TABLE_FADT));
 
         /* Setup FADT header and DSDT/FACS addresses */
 
@@ -484,18 +521,16 @@ AeBuildLocalTables (
 
         LocalFADT.XPm1bEventBlock.SpaceId = ACPI_ADR_SPACE_SYSTEM_IO;
         LocalFADT.XPm1bEventBlock.Address = LocalFADT.Pm1bEventBlock;
-        LocalFADT.XPm1bEventBlock.BitWidth = (UINT8) ACPI_MUL_8 (LocalFADT.Pm1EventLength);
-
-        /* Complete the FADT with the checksum */
-
-        LocalFADT.Header.Checksum = 0;
-        LocalFADT.Header.Checksum = (UINT8) -AcpiTbChecksum (
-            (void *) &LocalFADT, LocalFADT.Header.Length);
+        LocalFADT.XPm1bEventBlock.BitWidth = (UINT8)
+            ACPI_MUL_8 (LocalFADT.Pm1EventLength);
     }
 
+    AeInitializeTableHeader ((void *) &LocalFADT,
+        ACPI_SIG_FADT, sizeof (ACPI_TABLE_FADT));
+
     /* Build a FACS */
 
-    ACPI_MEMSET (&LocalFACS, 0, sizeof (ACPI_TABLE_FACS));
+    memset (&LocalFACS, 0, sizeof (ACPI_TABLE_FACS));
     ACPI_MOVE_NAME (LocalFACS.Signature, ACPI_SIG_FACS);
 
     LocalFACS.Length = sizeof (ACPI_TABLE_FACS);
@@ -509,11 +544,13 @@ AeBuildLocalTables (
          * Build a fake table [TEST] so that we make sure that the
          * ACPICA core ignores it
          */
-        ACPI_MEMSET (&LocalTEST, 0, sizeof (ACPI_TABLE_HEADER));
+        memset (&LocalTEST, 0, sizeof (ACPI_TABLE_HEADER));
         ACPI_MOVE_NAME (LocalTEST.Signature, "TEST");
 
         LocalTEST.Revision = 1;
         LocalTEST.Length = sizeof (ACPI_TABLE_HEADER);
+
+        LocalTEST.Checksum = 0;
         LocalTEST.Checksum = (UINT8) -AcpiTbChecksum (
             (void *) &LocalTEST, LocalTEST.Length);
 
@@ -521,11 +558,13 @@ AeBuildLocalTables (
          * Build a fake table with a bad signature [BAD!] so that we make
          * sure that the ACPICA core ignores it
          */
-        ACPI_MEMSET (&LocalBADTABLE, 0, sizeof (ACPI_TABLE_HEADER));
+        memset (&LocalBADTABLE, 0, sizeof (ACPI_TABLE_HEADER));
         ACPI_MOVE_NAME (LocalBADTABLE.Signature, "BAD!");
 
         LocalBADTABLE.Revision = 1;
         LocalBADTABLE.Length = sizeof (ACPI_TABLE_HEADER);
+
+        LocalBADTABLE.Checksum = 0;
         LocalBADTABLE.Checksum = (UINT8) -AcpiTbChecksum (
             (void *) &LocalBADTABLE, LocalBADTABLE.Length);
     }
@@ -553,16 +592,83 @@ AeInstallTables (
     ACPI_STATUS             Status;
     ACPI_TABLE_HEADER       Header;
     ACPI_TABLE_HEADER       *Table;
+    UINT32                  i;
 
 
-    Status = AcpiInitializeTables (Tables, ACPI_MAX_INIT_TABLES, TRUE);
-    AE_CHECK_OK (AcpiInitializeTables, Status);
+    Status = AcpiInitializeTables (NULL, ACPI_MAX_INIT_TABLES, TRUE);
+    ACPI_CHECK_OK (AcpiInitializeTables, Status);
+
+    /*
+     * The following code is prepared to test the deferred table
+     * verification mechanism. When AcpiGbl_EnableTableValidation is set
+     * to FALSE by default, AcpiReallocateRootTable() sets it back to TRUE
+     * and triggers the deferred table verification mechanism accordingly.
+     */
+    (void) AcpiReallocateRootTable ();
+
+    if (AcpiGbl_LoadTestTables)
+    {
+        /* Test multiple table/UEFI support. First, get the headers */
+
+        Status = AcpiGetTableHeader (ACPI_SIG_UEFI, 1, &Header);
+        ACPI_CHECK_OK (AcpiGetTableHeader, Status);
+
+        Status = AcpiGetTableHeader (ACPI_SIG_UEFI, 2, &Header);
+        ACPI_CHECK_OK (AcpiGetTableHeader, Status);
+
+        Status = AcpiGetTableHeader (ACPI_SIG_UEFI, 3, &Header);
+        ACPI_CHECK_STATUS (AcpiGetTableHeader, Status, AE_NOT_FOUND);
+
+        /* Now get the actual tables */
+
+        Status = AcpiGetTable (ACPI_SIG_UEFI, 1, &Table);
+        ACPI_CHECK_OK (AcpiGetTable, Status);
+
+        Status = AcpiGetTable (ACPI_SIG_UEFI, 2, &Table);
+        ACPI_CHECK_OK (AcpiGetTable, Status);
+
+        Status = AcpiGetTable (ACPI_SIG_UEFI, 3, &Table);
+        ACPI_CHECK_STATUS (AcpiGetTable, Status, AE_NOT_FOUND);
+    }
+
+    /* Check that we can get all of the ACPI tables */
+
+    for (i = 0; ; i++)
+    {
+        Status = AcpiGetTableByIndex (i, &Table);
+        if ((Status == AE_BAD_PARAMETER) || !Table)
+        {
+            break;
+        }
+
+        ACPI_CHECK_OK (AcpiGetTableByIndex, Status);
+    }
+
+    return (AE_OK);
+}
+
+
+/******************************************************************************
+ *
+ * FUNCTION:    AeLoadTables
+ *
+ * PARAMETERS:  None
+ *
+ * RETURN:      Status
+ *
+ * DESCRIPTION: Load the definition block ACPI tables
+ *
+ *****************************************************************************/
+
+ACPI_STATUS
+AeLoadTables (
+    void)
+{
+    ACPI_STATUS             Status;
 
-    Status = AcpiReallocateRootTable ();
-    AE_CHECK_OK (AcpiReallocateRootTable, Status);
 
     Status = AcpiLoadTables ();
-    AE_CHECK_OK (AcpiLoadTables, Status);
+    ACPI_CHECK_OK (AcpiLoadTables, Status);
 
     /*
      * Test run-time control method installation. Do it twice to test code
@@ -582,31 +688,6 @@ AeInstallTables (
             AcpiFormatException (Status));
     }
 
-    if (AcpiGbl_LoadTestTables)
-    {
-        /* Test multiple table/UEFI support. First, get the headers */
-
-        Status = AcpiGetTableHeader (ACPI_SIG_UEFI, 1, &Header);
-        AE_CHECK_OK (AcpiGetTableHeader, Status);
-
-        Status = AcpiGetTableHeader (ACPI_SIG_UEFI, 2, &Header);
-        AE_CHECK_OK (AcpiGetTableHeader, Status);
-
-        Status = AcpiGetTableHeader (ACPI_SIG_UEFI, 3, &Header);
-        AE_CHECK_STATUS (AcpiGetTableHeader, Status, AE_NOT_FOUND);
-
-        /* Now get the actual tables */
-
-        Status = AcpiGetTable (ACPI_SIG_UEFI, 1, &Table);
-        AE_CHECK_OK (AcpiGetTable, Status);
-
-        Status = AcpiGetTable (ACPI_SIG_UEFI, 2, &Table);
-        AE_CHECK_OK (AcpiGetTable, Status);
-
-        Status = AcpiGetTable (ACPI_SIG_UEFI, 3, &Table);
-        AE_CHECK_STATUS (AcpiGetTable, Status, AE_NOT_FOUND);
-    }
-
     return (AE_OK);
 }
 
@@ -630,5 +711,5 @@ AcpiOsGetRootPointer (
     void)
 {
 
-    return ((ACPI_PHYSICAL_ADDRESS) &LocalRSDP);
+    return (ACPI_PTR_TO_PHYSADDR (&LocalRSDP));
 }
