@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2013, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,6 +111,42 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #include "acpinames.h"
@@ -120,9 +156,12 @@
 
 /* Local prototypes */
 
-ACPI_PHYSICAL_ADDRESS
-AeLocalGetRootPointer (
-    void);
+static void
+AnInitializeTableHeader (
+    ACPI_TABLE_HEADER       *Header,
+    char                    *Signature,
+    UINT32                  Length);
+
 
 /* Non-AML tables that are constructed locally and installed */
 
@@ -145,12 +184,48 @@ static ACPI_TABLE_XSDT          *LocalXSDT;
 #define BASE_XSDT_SIZE          (sizeof (ACPI_TABLE_XSDT) + \
                                     ((BASE_XSDT_TABLES -1) * sizeof (UINT64)))
 
-ACPI_TABLE_DESC                 Tables[ACPI_MAX_INIT_TABLES];
+
+/******************************************************************************
+ *
+ * FUNCTION:    AnInitializeTableHeader
+ *
+ * PARAMETERS:  Header          - A valid standard ACPI table header
+ *              Signature       - Signature to insert
+ *              Length          - Length of the table
+ *
+ * RETURN:      None. Header is modified.
+ *
+ * DESCRIPTION: Initialize the table header for a local ACPI table.
+ *
+ *****************************************************************************/
+
+static void
+AnInitializeTableHeader (
+    ACPI_TABLE_HEADER       *Header,
+    char                    *Signature,
+    UINT32                  Length)
+{
+
+    ACPI_MOVE_NAME (Header->Signature, Signature);
+    Header->Length = Length;
+
+    Header->OemRevision = 0x1001;
+    strncpy (Header->OemId, "Intel", ACPI_OEM_ID_SIZE);
+    strncpy (Header->OemTableId, "AcpiName", ACPI_OEM_TABLE_ID_SIZE);
+    strncpy (Header->AslCompilerId, "INTL", ACPI_NAME_SIZE);
+    Header->AslCompilerRevision = ACPI_CA_VERSION;
+
+    /* Set the checksum, must set to zero first */
+
+    Header->Checksum = 0;
+    Header->Checksum = (UINT8) -AcpiTbChecksum (
+        (void *) Header, Header->Length);
+}
 
 
 /******************************************************************************
  *
- * FUNCTION:    AeBuildLocalTables
+ * FUNCTION:    AnBuildLocalTables
  *
  * PARAMETERS:  TableCount      - Number of tables on the command line
  *              TableList       - List of actual tables from files
@@ -163,30 +238,31 @@ ACPI_TABLE_DESC                 Tables[ACPI_MAX_INIT_TABLES];
  *****************************************************************************/
 
 ACPI_STATUS
-AeBuildLocalTables (
-    UINT32                  TableCount,
-    AE_TABLE_DESC           *TableList)
+AnBuildLocalTables (
+    ACPI_NEW_TABLE_DESC     *TableList)
 {
+    UINT32                  TableCount = 0;
     ACPI_PHYSICAL_ADDRESS   DsdtAddress = 0;
     UINT32                  XsdtSize;
-    AE_TABLE_DESC           *NextTable;
+    ACPI_NEW_TABLE_DESC     *NextTable;
     UINT32                  NextIndex;
     ACPI_TABLE_FADT         *ExternalFadt = NULL;
 
 
     /*
-     * Update the table count. For DSDT, it is not put into the XSDT. For
-     * FADT, this is already accounted for since we usually install a
-     * local FADT.
+     * Update the table count. For the DSDT, it is not put into the XSDT.
+     * For the FADT, this table is already accounted for since we usually
+     * install a local FADT.
      */
     NextTable = TableList;
     while (NextTable)
     {
-        if (ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_DSDT) ||
-            ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_FADT))
+        if (!ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_DSDT) &&
+            !ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_FADT))
         {
-            TableCount--;
+            TableCount++;
         }
+
         NextTable = NextTable->Next;
     }
 
@@ -200,16 +276,15 @@ AeBuildLocalTables (
         return (AE_NO_MEMORY);
     }
 
-    ACPI_MEMSET (LocalXSDT, 0, XsdtSize);
-    ACPI_MOVE_NAME (LocalXSDT->Header.Signature, ACPI_SIG_XSDT);
-    LocalXSDT->Header.Length = XsdtSize;
-    LocalXSDT->Header.Revision = 1;
-
+    memset (LocalXSDT, 0, XsdtSize);
     LocalXSDT->TableOffsetEntry[0] = ACPI_PTR_TO_PHYSADDR (&LocalFADT);
 
     /*
      * Install the user tables. The DSDT must be installed in the FADT.
      * All other tables are installed directly into the XSDT.
+     *
+     * Note: The tables are loaded in reverse order from the incoming
+     * input, which makes it match the command line order.
      */
     NextIndex = BASE_XSDT_TABLES;
     NextTable = TableList;
@@ -233,33 +308,38 @@ AeBuildLocalTables (
         }
         else if (ACPI_COMPARE_NAME (NextTable->Table->Signature, ACPI_SIG_FADT))
         {
-            ExternalFadt = ACPI_CAST_PTR (ACPI_TABLE_FADT, NextTable->Table);
-            LocalXSDT->TableOffsetEntry[2] = ACPI_PTR_TO_PHYSADDR (NextTable->Table);
+            ExternalFadt =
+                ACPI_CAST_PTR (ACPI_TABLE_FADT, NextTable->Table);
+            LocalXSDT->TableOffsetEntry[0] =
+                ACPI_PTR_TO_PHYSADDR (NextTable->Table);
         }
         else
         {
             /* Install the table in the XSDT */
 
-            LocalXSDT->TableOffsetEntry[NextIndex] = ACPI_PTR_TO_PHYSADDR (NextTable->Table);
+            LocalXSDT->TableOffsetEntry[TableCount - NextIndex + 1] =
+                ACPI_PTR_TO_PHYSADDR (NextTable->Table);
             NextIndex++;
         }
 
         NextTable = NextTable->Next;
     }
 
-    /* Build an RSDP */
+    /* Build an RSDP. Contains a valid XSDT only, no RSDT */
+
+    memset (&LocalRSDP, 0, sizeof (ACPI_TABLE_RSDP));
+    ACPI_MAKE_RSDP_SIG (LocalRSDP.Signature);
+    memcpy (LocalRSDP.OemId, "Intel", 6);
 
-    ACPI_MEMSET (&LocalRSDP, 0, sizeof (ACPI_TABLE_RSDP));
-    ACPI_MEMCPY (LocalRSDP.Signature, ACPI_SIG_RSDP, 8);
-    ACPI_MEMCPY (LocalRSDP.OemId, "I_TEST", 6);
     LocalRSDP.Revision = 2;
     LocalRSDP.XsdtPhysicalAddress = ACPI_PTR_TO_PHYSADDR (LocalXSDT);
     LocalRSDP.Length = sizeof (ACPI_TABLE_XSDT);
 
     /* Set checksums for both XSDT and RSDP */
 
-    LocalXSDT->Header.Checksum = (UINT8) -AcpiTbChecksum (
-        (void *) LocalXSDT, LocalXSDT->Header.Length);
+    AnInitializeTableHeader ((void *) LocalXSDT, ACPI_SIG_XSDT, XsdtSize);
+
+    LocalRSDP.Checksum = 0;
     LocalRSDP.Checksum = (UINT8) -AcpiTbChecksum (
         (void *) &LocalRSDP, ACPI_RSDP_CHECKSUM_LENGTH);
 
@@ -268,21 +348,36 @@ AeBuildLocalTables (
         return (AE_SUPPORT);
     }
 
+    /*
+     * Build an FADT. There are two options for the FADT:
+     * 1) Incoming external FADT specified on the command line
+     * 2) A fully featured local FADT
+     */
+    memset (&LocalFADT, 0, sizeof (ACPI_TABLE_FADT));
+
     if (ExternalFadt)
     {
         /*
-         * Use the external FADT, but we must update the DSDT/FACS addresses
-         * as well as the checksum
+         * Use the external FADT, but we must update the DSDT/FACS
+         * addresses as well as the checksum
          */
-        ExternalFadt->Dsdt = DsdtAddress;
+        ExternalFadt->Dsdt = (UINT32) DsdtAddress;
         ExternalFadt->Facs = ACPI_PTR_TO_PHYSADDR (&LocalFACS);
 
-        if (ExternalFadt->Header.Length > ACPI_PTR_DIFF (&ExternalFadt->XDsdt, ExternalFadt))
+        /*
+         * If there room in the FADT for the XDsdt and XFacs 64-bit
+         * pointers, use them.
+         */
+        if (ExternalFadt->Header.Length > ACPI_PTR_DIFF (
+            &ExternalFadt->XDsdt, ExternalFadt))
         {
+            ExternalFadt->Dsdt = 0;
+            ExternalFadt->Facs = 0;
             ExternalFadt->XDsdt = DsdtAddress;
             ExternalFadt->XFacs = ACPI_PTR_TO_PHYSADDR (&LocalFACS);
         }
-        /* Complete the FADT with the checksum */
+
+        /* Complete the external FADT with the checksum */
 
         ExternalFadt->Header.Checksum = 0;
         ExternalFadt->Header.Checksum = (UINT8) -AcpiTbChecksum (
@@ -293,8 +388,7 @@ AeBuildLocalTables (
         /*
          * Build a local FADT so we can test the hardware/event init
          */
-        ACPI_MEMSET (&LocalFADT, 0, sizeof (ACPI_TABLE_FADT));
-        ACPI_MOVE_NAME (LocalFADT.Header.Signature, ACPI_SIG_FADT);
+        LocalFADT.Header.Revision = 5;
 
         /* Setup FADT header and DSDT/FACS addresses */
 
@@ -304,9 +398,6 @@ AeBuildLocalTables (
         LocalFADT.XDsdt = DsdtAddress;
         LocalFADT.XFacs = ACPI_PTR_TO_PHYSADDR (&LocalFACS);
 
-        LocalFADT.Header.Revision = 3;
-        LocalFADT.Header.Length = sizeof (ACPI_TABLE_FADT);
-
         /* Miscellaneous FADT fields */
 
         LocalFADT.Gpe0BlockLength = 16;
@@ -333,30 +424,27 @@ AeBuildLocalTables (
 
         LocalFADT.XPm1bEventBlock.SpaceId = ACPI_ADR_SPACE_SYSTEM_IO;
         LocalFADT.XPm1bEventBlock.Address = LocalFADT.Pm1bEventBlock;
-        LocalFADT.XPm1bEventBlock.BitWidth = (UINT8) ACPI_MUL_8 (LocalFADT.Pm1EventLength);
-
-        /* Complete the FADT with the checksum */
-
-        LocalFADT.Header.Checksum = 0;
-        LocalFADT.Header.Checksum = (UINT8) -AcpiTbChecksum (
-            (void *) &LocalFADT, LocalFADT.Header.Length);
+        LocalFADT.XPm1bEventBlock.BitWidth = (UINT8)
+            ACPI_MUL_8 (LocalFADT.Pm1EventLength);
     }
 
+    AnInitializeTableHeader ((void *) &LocalFADT,
+        ACPI_SIG_FADT, sizeof (ACPI_TABLE_FADT));
+
     /* Build a FACS */
 
-    ACPI_MEMSET (&LocalFACS, 0, sizeof (ACPI_TABLE_FACS));
+    memset (&LocalFACS, 0, sizeof (ACPI_TABLE_FACS));
     ACPI_MOVE_NAME (LocalFACS.Signature, ACPI_SIG_FACS);
 
     LocalFACS.Length = sizeof (ACPI_TABLE_FACS);
     LocalFACS.GlobalLock = 0x11AA0011;
-
     return (AE_OK);
 }
 
 
 /******************************************************************************
  *
- * FUNCTION:    AeLocalGetRootPointer
+ * FUNCTION:    AcpiOsGetRootPointer
  *
  * PARAMETERS:  None
  *
@@ -368,9 +456,9 @@ AeBuildLocalTables (
  *****************************************************************************/
 
 ACPI_PHYSICAL_ADDRESS
-AeLocalGetRootPointer (
+AcpiOsGetRootPointer (
     void)
 {
 
-    return ((ACPI_PHYSICAL_ADDRESS) &LocalRSDP);
+    return (ACPI_PTR_TO_PHYSADDR (&LocalRSDP));
 }
